#coding=utf-8

import os
import sys, traceback
from .IPythonApiClient import IPythonApiClient as RPCClient

__all__ = [
    'create_extend_data'
    , 'create_external_data'
    , 'get_extend_data'
    , 'get_extend_data_list'
    , 'get_extend_data_value'
    , 'get_external_data'
    , 'get_external_data_list'
    , 'reset_extend_data_stock_list'
    , 'reset_extend_data_time_list'
    , 'set_extend_data'
    , 'set_extend_data_value'
]

CLIENT = None

from os.path import abspath, dirname
__curdir = dirname(abspath(__file__))

__rpc_config = __curdir + '/xtdata.ini'
__xtdata_config = __curdir + '/xtdata.ini'

from .IPythonApiClient import rpc_init
__rpc_init_status = rpc_init(__rpc_config)
if __rpc_init_status < 0:
    print(f'rpc初始化失败，配置文件：{__rpc_config}')

def get_client():
    global CLIENT
    if not CLIENT:
        CLIENT = RPCClient('client_xtdata', __xtdata_config)
        if not CLIENT:
            raise Exception("创建连接失败！")
    if not CLIENT.is_connected():
        if not CLIENT.connect():
            raise Exception("无法连接行情服务！")
    return CLIENT

def get_extend_data_value(name, stock, time):
    '''
    获取某个时间某支股票的扩展数据结果
    :param name: 扩展数据名
    :param stock: 股票代码 e.g. "SH600000"
    :param time: 毫秒数的时间戳
    :return: double 扩展数据结果
    '''
    client = get_client()
    return client.get_extend_data_value(name, stock, time)
    
def get_extend_data_list():
    '''
    获取扩展数据列表
    : return: list[str] 
    '''
    client = get_client()
    return client.get_extend_data_list()
    
def create_extend_data(name, overwrite):
    '''
    新增扩展数据
    :param name: 扩展数据名
    :param overwrite: 是否覆盖 True：覆盖 Fales: e.g. name为"test"时 test已存在，创建test1
    :return: str 创建的扩展数据名
    '''
    client = get_client()
    return client.create_extend_data(name, overwrite)
    
def set_extend_data_value(name, timetag, values):
    '''
    修改某个时间的扩展数据结果
    :param name: 扩展数据名
    :param timetag: 毫秒数的时间戳
    :param values: 填充的数据 e.g. {"SH600000": 1.23, "SH600004":1.23}
    '''
    client = get_client()
    return client.set_extend_data_value(name, timetag, values)
    
def set_extend_data(name, datas):
    '''
    修改某个时间的扩展数据结果
    :param name: 扩展数据名
    :param datas: 填充的数据 e.g. {1570550400000: {"SH600000": 1.23, "SH600004":1.23}}
    '''
    client = get_client()
    return client.set_extend_data(name, datas)
    
def get_extend_data(name, stockList):
    '''
    获取扩展数据内容
    :param name: 扩展数据名
    :param stockList: 筛选用的合约列表 ["600000.SH" ...],为空时不会有数据返回
    :return: dict e.g. {1570550400000: {"600000.SH": 1.23, "600004.SH":1.23}}
    '''
    client = get_client()
    return client.get_extend_data(name, stockList)
    
def reset_extend_data_stock_list(name, stockList):
    '''
    设置扩展数据的股票列表
    :param name: 扩展数据名
    Lparam stockList: list[str] 股票列表
    '''
    client = get_client()
    return client.reset_extend_data_stock_list(name, stockList)
    
def reset_extend_data_time_list(name, timeList):
    '''
    设置扩展数据的时间列表
    :param name: 扩展数据名
    Lparam timeList: list[str] 时间列表
    '''
    client = get_client()
    return client.reset_extend_data_time_list(name, timeList)
    
def get_external_data_list():
    '''
    获取外部数据列表
    : return: list[str] 
    '''
    client = get_client()
    return client.get_external_data_list()
    
def create_external_data(name, period, data):
    '''
    生成外部数据，如果已存在则覆盖
    :param name: 外部数据名
    :param period: 周期 e.g. 86400000
    :param data: 数据 e.g. {1593273600000: 3.5, 1593360000000: 1.5}
    :return: str 创建的外部数据名
    '''
    client = get_client()
    return client.create_external_data(name, period, data)
    
def get_external_data(name):
    '''
    查询外部数据
    :param name: 外部数据名
    :return: dict{long long : double} 
    '''
    client = get_client()
    return client.get_external_data(name)
    